//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 2000  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// TutorialGUI class - creates a frame to display a tutorial
//
import java.awt.Frame;
import java.awt.Font;
import java.awt.Panel;
import java.awt.BorderLayout;
import java.awt.Label;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.MenuItem;
import java.awt.MenuShortcut;
import java.awt.TextArea;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.ResourceBundle;
import java.text.MessageFormat;

/**
 * Creates a frame to display a tutorial.
 */
public class TutorialGUI extends Frame
		implements ActionListener, WindowListener {
	private boolean inAnApplet = true;

	// message and colour catalog information
	private static final ResourceBundle	messages	=
				ResourceBundle.getBundle( "MessageResource" );
	private static final ResourceBundle	colours		=
				ResourceBundle.getBundle( "ColorResource" );

	// menubar GUI components
	private MenuBar		menu_bar;
	private Menu		file_menu, help_menu, history_menu;
	private MenuItem	file_close;

	// GUI components
	private Label		title_label;
	private TextArea	text_area;

	// lesson history
	private History		history;
	private Hashtable	history_menuitems;

	/**
	 * Initialize the whole GUI set.
	 */
	public TutorialGUI( boolean inAnApplet ) {
		this.addWindowListener( this );
		this.inAnApplet = inAnApplet;

		// create GUI components
		setTitle( messages.getString( "TutorialGUI.title" ));
		setLayout( new BorderLayout() );
		setForeground( Colour.find_colour(
			colours.getString( "TutorialGUI.foreground" )));
		setBackground( Colour.find_colour(
			colours.getString( "TutorialGUI.background" )));
		setFont( Font.decode(
			colours.getString( "TutorialGUI.font" )));

		// create the menu bar stuff
		menu_bar = new MenuBar();
		setMenuBar( menu_bar );
		// File menu
		file_menu = new Menu(
			messages.getString( "TutorialGUI.file" ));
		menu_bar.add( file_menu );
		file_close = new MenuItem(
			messages.getString( "TutorialGUI.file_close" ),
			new MenuShortcut(
				messages.getString(
				"TutorialGUI.file_close_sc" ).charAt(0) ));
		file_close.addActionListener( this );
		file_menu.add( file_close );
		// History menu
		history_menu = new Menu(
			messages.getString( "TutorialGUI.history" ));
		menu_bar.add( history_menu );
		history = new History( Integer.parseInt(
			messages.getString( "TutorialGUI.historylength" )));
		history_menuitems = new Hashtable();

		// create the label and text display
		title_label = new Label();
		add( "North", title_label );

		text_area = new TextArea(
				Integer.parseInt(
			messages.getString( "TutorialGUI.rows" )),
				Integer.parseInt(
			messages.getString( "TutorialGUI.columns" )));
		text_area.setForeground( Colour.find_colour(
			colours.getString( "TutorialGUI.panel_foreground" )));
		text_area.setBackground( Colour.find_colour(
			colours.getString( "TutorialGUI.panel_background" )));
		text_area.setFont( Font.decode(
			colours.getString( "TutorialGUI.panel_font" )));
		text_area.setEditable( false );
		add( "Center", text_area );
	}


	/*
	 * Update the history menu with this lesson.
	 */
	private void update_history( TutorialLesson lesson ) {

		// get the entry format
		String entry_format	= messages.getString(
						"TutorialGUI.historyformat" );

		// see if this lesson is a new one to the list
		if ( ! history.remembers( lesson )) {

			// add to the lesson history
			history.remember( lesson );

			// new lesson, so update the history menu
			history_menu.removeAll();
			history_menuitems.clear();
			for ( Enumeration enum = history.relate();
					enum.hasMoreElements(); ) {

				// get the lesson title, and make a menu item
				Lesson hlesson = (Lesson)(enum.nextElement());
				String stext	= hlesson.get_text()
							.replace( '\n', ' ' )
							.replace( '\t', ' ' );
				String[] fmtargs= { hlesson.get_title(),
					( stext.length() > 20 ) ?
						stext.substring( 0, 20 ) :
									stext };
				MenuItem menuitem = new MenuItem(
					MessageFormat.format( entry_format,
								fmtargs ));
				menuitem.addActionListener( this );

				// add this menu item to the menu and list
				history_menu.add( menuitem );
				history_menuitems.put( menuitem, hlesson );
			}
		}
	}


	/**
	 * Display the tutorial text from a lesson passed in.
	 */
	public void reveal( TutorialLesson lesson ) {

		// update the lesson history menu
		update_history( lesson );

		// set up the title label and text area
		title_label.setText( lesson.get_title() );
		text_area.setText( lesson.get_text() );
		setVisible( true );
	}

	/*
	 * Handle dismiss, close, etc by just hiding ourselves.
	 */
	private void handle_close_button() {

		// simply just hide this frame from view
		setVisible( false );
	}


	/*
	 * Handle GUI component actions and events.
	 */
	public void actionPerformed( ActionEvent evt ) {
		if ( evt.getSource() instanceof MenuItem ) {
			if ( evt.getSource() == file_close ) {
				handle_close_button();
			}
			Lesson lesson = (Lesson)(history_menuitems.get(
							evt.getSource() ));
			if ( lesson != null
					&& lesson instanceof TutorialLesson ) {
				reveal( (TutorialLesson)lesson );
			}
		}
	}
	public void windowClosing( WindowEvent evt ) {
		handle_close_button();
	}
	public void windowClosed( WindowEvent evt ) {}
	public void windowOpened( WindowEvent evt ) {}
	public void windowIconified( WindowEvent evt ) {}
	public void windowDeiconified( WindowEvent evt ) {}
	public void windowActivated( WindowEvent evt ) {}
	public void windowDeactivated( WindowEvent evt ) {}


	/**
	 * Class test code.
	 */
	public static void main( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );

		TutorialGUI tutorial = new TutorialGUI( false );
		tutorial.pack();
		tutorial.reveal( new TutorialLesson ( "Test lesson title",
			"This is test text for a tutorial window" ));
	}
}
